<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="../../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:c="http://CraneSoftwrights.com/ns/parseCSV"
                exclude-result-prefixes="xs xsd c"
  version="2.0">

<xs:doc info="$Id: Crane-csv2gc.xsl,v 1.6 2013/01/18 23:00:01 admin Exp $"
        filename="Crane-csv2gc.xsl" vocabulary="DocBook">
  <xs:title>Crane-csv2gc - convert a CSV file into a simple genericode file</xs:title>
  <para>
    This converts a CSV file following conventions of the first few rows
    into a genericode file conforming to
    <ulink url="http://docs.oasis-open.org/codelist/cs-genericode-1.0/"
      >http://docs.oasis-open.org/codelist/cs-genericode-1.0/</ulink>.
  </para>
  <section>
    <title>Invocation</title>
    <para>Using a package such as SaxonHE <ulink url="http://saxon.sf.net"
    ><literal>http://saxon.sf.net</literal></ulink>, one can invoke this
    stylesheet along the lines of:</para>
    <programlisting>java -Xss2m -Xms128m -Xmx512m -jar saxon9he.jar -it:start Crane-csv2gc.xsl "csvFile=test.csv" >test.gc</programlisting>
    <para>
      It is convenient to invoke a schema validation of the result to ensure
      critical information is not missed in the file.
    </para>
  </section>
</xs:doc>
  
<xsl:include href="Crane-ParseCSV.xsl"/>

<xs:param ignore-ns="yes">
  <para>The filename of the input CSV information.</para>
  <para>
    The first few records of this CSV file are critically important and
    not found in typical CSV files exported from spreadsheet or database
    software.  However, there should only be six additional records
    placed before the start of the exported CSV files.
  </para>
  <para>
    It is assumed that all language identifiers are "en" for English.
  </para>
  <itemizedlist>
    <listitem>Record 1 - list-level metadata identification information
      in order
      (values ending in "?" can be left empty to omit the metadata)
      <itemizedlist>
        <listitem>short name - e.g. PackagingTypeCode</listitem>
        <listitem>long name - e.g. Packaging Type</listitem>
        <listitem>list identifier - e.g. UN/ECE rec 20</listitem>
        <listitem>version - e.g. 9e</listitem>
        <listitem>canonical URI - e.g. urn:oasis:names:specification:ubl:codelist:gc:PackagingTypeCode</listitem>
        <listitem>canonical version URI - e.g. urn:oasis:names:specification:ubl:codelist:gc:PackagingTypeCode-2.0-update</listitem>
        <listitem>location URI? - e.g. http://docs.oasis-open.org/ubl/os-UBL-2.0-update/cl/gc/default/PackagingTypeCode-2.0.gc</listitem>
        <listitem>alternate format location uri - e.g. http://www.unece.org/fileadmin/DAM/cefact/recommendations/rec21/rec21_Rev9e_2012.xls</listitem>
        <listitem>alternate format location MIME type - e.g. application/vnd.ms-excel</listitem>
        <listitem>ageny name? - e.g. United Nations Economic Commission for Europe</listitem>
        <listitem>agency identifier? - e.g. 6</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Record 2 - column identifiers ordered corresponding to row columns, 
      adding an asterisk "*" at the end of the identifier that is the key, plus
      a single key identifier at the end;
      leave an identifier empty in order to ignore the column in the rows
      <itemizedlist>
        <listitem>e.g. status,code*,name,description,numeric,codeKey</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Record 3 - column short names ordered corresponding to row columns, plus
      a single key short name at the end
      <itemizedlist>
        <listitem>e.g. Status,Code,Name,Description,NumericCode,CodeKey</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Record 4 - column cardinality as "optional" or "required"
      <itemizedlist>
        <listitem>e.g. optional,required,required,optional,optional</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Record 5 - column data types ordered corresponding to row columns
      <itemizedlist>
        <listitem>e.g. normalizedString,normalizedString,string,string,string</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Record 6 - column long names ordered corresponding to row columns;
      these are typically the column headers of the table that were exported
      to make the CSV file; this line can be left entirely blank to skip
      column long names
      <itemizedlist>
        <listitem>e.g. Code,Name,Description Field</listitem>
      </itemizedlist>
    </listitem>
    <listitem>
      Records 7 and on - rows of coded values and associated value-level
      metadata (empty input values do not create output values);
      these are typically the rows of the table that were exported
      to make the CSV file
      <itemizedlist>
<listitem>e.g. ,1A,"Drum, steel",,34</listitem>
<listitem>e.g. ,1B,"Drum, aluminium",,34</listitem>
<listitem>e.g. ,1D,"Drum, plywood",,34</listitem>
<listitem>e.g. ,1F,"Container, flexible",A packaging container of flexible construction.,93</listitem>
<listitem>e.g. X,SX,Set,,11 to 45,? To be removed from list?</listitem>
      </itemizedlist>
    </listitem>
  </itemizedlist>
</xs:param>
<xsl:param name="csvFile" as="xsd:string" required="yes"/>

<xs:output>
  <para>A genericode file is most easily reviewed when indented.</para>
</xs:output>
<xsl:output indent="yes"/>

<xs:template>
  <para>
    All work is done in one pull-styled template, walking over the CSV file.
  </para>
</xs:template>
<xsl:template name="start" match="/">
  <!--first get the information from the input file-->
  <!--start the genericode file-->
  <gc:CodeList 
    xmlns:gc="http://docs.oasis-open.org/codelist/ns/genericode/1.0/">
    <xsl:call-template name="c:csv2gcContent">
      <xsl:with-param name="csvFile" select="$csvFile"/>
    </xsl:call-template>
  </gc:CodeList>
</xsl:template>

<xs:template>
  <para>
    The content template is separate in order to be exploited by an
    importing stylesheet.
  </para>
  <xs:param name="csvFile">
    <para>The file name to be parsed</para>
  </xs:param>
  <xs:param name="encoding">
    <para>The encoding of the file name to be parsed</para>
  </xs:param>
</xs:template>
<xsl:template name="c:csv2gcContent"
  xmlns:gc="http://docs.oasis-open.org/codelist/ns/genericode/1.0/">
  <xsl:param name="csvFile" as="xsd:string"/>
  <xsl:param name="encoding" select="'iso-8859-1'" as="xsd:string"/>
  <xsl:variable name="records" as="element(record)+">
    <xsl:call-template name="c:parseCSV">
      <xsl:with-param name="filename" 
                      select="resolve-uri($csvFile,static-base-uri())"/>
      <xsl:with-param name="encoding" select="$encoding"/>
      <xsl:with-param name="xlate-eol" select="true()"/>
    </xsl:call-template>
  </xsl:variable>
  <!--all front matter from first 6 records-->
  <Identification>
    <!--all identification information in the first record-->
    <xsl:for-each select="$records[1]">
      <ShortName>
        <xsl:value-of select="field[1]"/>
      </ShortName>
      <LongName xml:lang="en">
        <xsl:value-of select="field[2]"/>
      </LongName>
      <xsl:for-each select="field[3][normalize-space(.)]">
        <LongName Identifier="listID">
          <xsl:value-of select="."/>
        </LongName>
      </xsl:for-each>
      <Version>
        <xsl:value-of select="field[4]"/>
      </Version>
      <CanonicalUri>
        <xsl:value-of select="field[5]"/>
      </CanonicalUri>
      <CanonicalVersionUri>
        <xsl:value-of select="field[6]"/>
      </CanonicalVersionUri>
      <LocationUri>
        <xsl:value-of select="field[7]"/>
      </LocationUri>
      <xsl:if
        test="normalize-space(field[8]) and 
                    normalize-space(field[9])">
        <AlternateFormatLocationUri MimeType="{field[9]}">
          <xsl:value-of select="field[8]"/>
        </AlternateFormatLocationUri>
      </xsl:if>
      <xsl:if
        test="normalize-space(field[10]) or 
                    normalize-space(field[11])">
        <Agency>
          <xsl:for-each select="field[10][normalize-space(.)]">
            <LongName xml:lang="en">
              <xsl:value-of select="."/>
            </LongName>
          </xsl:for-each>
          <xsl:for-each select="field[11][normalize-space(.)]">
            <Identifier>
              <xsl:value-of select="."/>
            </Identifier>
          </xsl:for-each>
        </Agency>
      </xsl:if>
    </xsl:for-each>
  </Identification>
  <!--all columns et information in records 2 through 6-->
  <ColumnSet>
    <xsl:for-each select="1 to count($records[2]/field) - 1">
      <xsl:variable name="this" select="position()"/>
      <xsl:if test="normalize-space($records[2]/field[$this])">
        <Column Id="{translate($records[2]/field[$this],'*','')}"
          Use="{$records[4]/field[$this]}">
          <ShortName>
            <xsl:value-of select="$records[3]/field[$this]"/>
          </ShortName>
          <xsl:for-each select="$records[6]/field[$this][normalize-space(.)]">
            <LongName xml:lang="en">
              <xsl:value-of select="."/>
            </LongName>
          </xsl:for-each>
          <Data Type="{$records[5]/field[$this]}"/>
        </Column>
      </xsl:if>
    </xsl:for-each>
    <Key Id="{$records[2]/field[last()]}">
      <ShortName>
        <xsl:value-of select="$records[3]/field[last()]"/>
      </ShortName>
      <ColumnRef Ref="{translate($records[2]/field[contains(.,'*')],'*','')}"
      />
    </Key>
  </ColumnSet>
  <!--the rest of the file makes up the rows of the list-->
  <xsl:if test="$records[7]">
    <SimpleCodeList>
      <xsl:for-each select="$records[position()>6]">
        <Row>
          <!--only look for fields that have a referenceable identifier-->
          <xsl:for-each select="field">
            <xsl:variable name="this" select="position()"/>
            <xsl:if
              test="normalize-space($records[2]/field[$this]) and
                          normalize-space(.)">
              <Value ColumnRef="{translate($records[2]/field[$this],'*','')}">
                <SimpleValue>
                  <xsl:value-of select="."/>
                </SimpleValue>
              </Value>
            </xsl:if>
          </xsl:for-each>
        </Row>
      </xsl:for-each>
    </SimpleCodeList>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>